#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "common.h"

char *logfile = NULL;
char *loguser = NULL;
char *pidfile = NULL;
int background = 0;

void usage() {
	die("usage: runservice [-b] [-l logfile] [-u loguser] [-p pidfile] "
		"command ...");
}

int main(int argc, char **argv) {
	char **args, **p;

	setprogname("runservice");

	while (1) {
		char c = getopt(argc, argv, "+bl:u:p:");
		if (c == -1)
			break;

		switch (c) {
		case 'b':
			background = 1;
			break;
		case 'l':
			logfile = strdup(optarg);
			break;
		case 'u':
			loguser = strdup(optarg);
			break;
		case 'p':
			pidfile = strdup(optarg);
			break;
		default:
			usage();
		}
	}

	if ((argc - optind) < 1)
		usage();

	args = malloc((16 + argc - optind) * sizeof *args);
	if (args == NULL)
		die("out of memory");

	p = args;
	if (background)
		*p++ = "background";
	if (logfile != NULL) {
		*p++ = "pipedto";
		*p++ = "-oe";
		*p++ = "reliably";
		if (pidfile != NULL) {
			*p++ = "-p";
			*p++ = strdup(join(pidfile, "-log", NULL));
		}
		if (loguser != NULL) {
			*p++ = "setuidgid";
			*p++ = loguser;
		}
		*p++ = "dumblog";
		*p++ = "-c";
		*p++ = logfile;
		*p++ = ";";
	}
	*p++ = "reliably";
	if (pidfile != NULL) {
		*p++ = "-p";
		*p++ = pidfile;
	}
	while (optind < argc)
		*p++ = argv[optind++];	
	*p++ = NULL;

	execvp(args[0], args);
	dies("execvp failed");
	return 0; /* NOTREACHED */
}

